#ifndef bqtBindingQbHH
#define bqtBindingQbHH

#include <string>
#include "symtables.hh"

/* Used by lexer */
void SetDefVarType(char begin, char end, char type);
void SetMetaDynamic();
void SetMetaStatic();

/* Used by parser */
char  HasTypeSuffix(const std::string& varname);

long double DeclareAsLabelInCurrentScope(const std::string& varname);
unsigned TranslateLabelToDataPtr(const long double l);

struct VarContext
{
    // Which sub is this var of? Empty = global.
    std::string SubName;

    // Pointer to the definition. 0 = not defined.
    ctxp info;
};

UtypeP FindUserType(const std::string& name);
bool DefineUserType(const std::string& name, UtypeP type);

exprp FindConstant(const std::string& name);
bool DefineConstant(const std::string& name, exprp p);

// Check if the identifier has been defined in the given class
VarContext FindScalarType(const std::string& name); // typesuffix optionally allowed
VarContext FindArrayType(const std::string& name); // typesuffix optionally allowed

bool ImplicitlyDefineScalar(const std::string& name); // typesuffix optionally allowed
bool ImplicitlyDefineArray(const std::string& name,
                              unsigned num_dimensions,
                              unsigned starting_index); // typesuffix optionally allowed

// The DIM statement.
bool ExplicitlyDefine(const std::string& name, TypeP type,
                      bool is_static, bool is_shared);

// Creates a type template for the use by DIM and Implicit definitions.
// Uses the type suffix to initialize the "Type" field. Leaves other fields at default.
TypeP CreateTypeTemplate(const std::string& name);



/* Note: Array definition may fail if a parametric FUNCTION exists by the same name.
 *       Scalar definition may fail if a CONST exists by the same name.
 *       Scalar definition may fail if a non-parametric FUNCTION exists by the same name.
 *
 *       CONST definition may fail if a scalar exists by the same name.
 *       CONST definition may fail if a non-parametric FUNCTION exists by the same name.
 *
 * Array definition will always check the $DYNAMIC state, but never allow redefinition.
 *
 * Note:
 *  DIM a!
 *  a% = 5   ' This generates a Duplicate definition
 *
 */

#endif
